==> structlog-0.1.0.tar.gz: structlog-0.1.0/tests/test_processors.py <==
# Copyright 2013 Hynek Schlawack
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import json

import pytest

from freezegun import freeze_time

import structlog

from structlog._compat import u
from structlog.processors import (
    JSONRenderer,
    KeyValueRenderer,
    TimeStamper,
    UnicodeEncoder,
    _JSONFallbackEncoder,
    format_exc_info,
)
from structlog.threadlocal import wrap_dict


@pytest.fixture
def event_dict():
    class A(object):
        def __repr__(self):
            return '<A(\o/)>'

    return {'a': A(), 'b': [3, 4], 'x': 7, 'y': 'test', 'z': (1, 2)}


def test_KeyValueRenderer(event_dict):
    assert (
        r"a=<A(\o/)> b=[3, 4] x=7 y='test' z=(1, 2)" ==
        KeyValueRenderer(sort_keys=True)(None, None, event_dict)
    )


class TestJSONRenderer(object):
    def test_renders_json(self, event_dict):
        assert (
            r'{"a": "<A(\\o/)>", "b": [3, 4], "x": 7, "y": "test", "z": '
            r'[1, 2]}'
            == JSONRenderer(sort_keys=True)(None, None, event_dict)
        )

    def test_FallbackEncoder_handles_ThreadLocalDictWrapped_dicts(self):
        s = json.dumps(wrap_dict(dict)({'a': 42}),
                       cls=_JSONFallbackEncoder)
        assert '{"a": 42}' == s

    def test_FallbackEncoder_falls_back(self):
        s = json.dumps({'date': datetime.date(1980, 3, 25)},
                       cls=_JSONFallbackEncoder,)

        assert '{"date": "datetime.date(1980, 3, 25)"}' == s


class TestTimeStamper(object):
    def test_disallowsNonUTCUNIXTimestamps(self):
        with pytest.raises(ValueError) as e:
            TimeStamper(utc=False)
        assert 'UNIX timestamps are always UTC.' == e.value.args[0]

    def test_insertsUTCUNIXTimestampByDefault(self):
        ts = TimeStamper()
        d = ts(None, None, {})
        # freezegun doesn't work with time.gmtime :(
        assert isinstance(d['timestamp'], int)

    @freeze_time('1980-03-25 16:00:00')
    def test_local(self):
        ts = TimeStamper(fmt='iso', utc=False)
        d = ts(None, None, {})
        assert '1980-03-25T16:00:00' == d['timestamp']

    @freeze_time('1980-03-25 16:00:00')
    def test_formats(self):
        ts = TimeStamper(fmt='%Y')
        d = ts(None, None, {})
        assert '1980' == d['timestamp']

    @freeze_time('1980-03-25 16:00:00')
    def test_adds_Z_to_iso(self):
        ts = TimeStamper(fmt='iso', utc=True)
        d = ts(None, None, {})
        assert '1980-03-25T16:00:00Z' == d['timestamp']


class TestFormatExcInfo(object):
    def test_formats_tuple(self, monkeypatch):
        monkeypatch.setattr(structlog.processors,
                            '_format_exception',
                            lambda exc_info: exc_info)
        d = format_exc_info(None, None, {'exc_info': (None, None, 42)})
        assert {'exception': (None, None, 42)} == d

    def test_gets_exc_info_on_bool(self):
        # monkeypatching sys.exc_info makes currently py.test return 1 on
        # success.
        try:
            raise ValueError('test')
        except ValueError:
            d = format_exc_info(None, None, {'exc_info': True})
        assert 'exc_info' not in d
        assert 'raise ValueError(\'test\')\nValueError: test' in d['exception']


class TestUnicodeEncoder(object):
    def test_encodes(self):
        ue = UnicodeEncoder()
        assert {'foo': b'b\xc3\xa4r'} == ue(None, None, {'foo': u('b\xe4r')})

    def test_passes_arguments(self):
        ue = UnicodeEncoder('latin1', 'xmlcharrefreplace')
        assert {'foo': b'&#8211;'} == ue(None, None, {'foo': u('\u2013')})

==> structlog-0.3.0.tar.gz: structlog-0.3.0/tests/test_processors.py <==
# Copyright 2013 Hynek Schlawack
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import json

import pytest

from freezegun import freeze_time

import structlog

from structlog._compat import u
from structlog.processors import (
    JSONRenderer,
    KeyValueRenderer,
    TimeStamper,
    UnicodeEncoder,
    _JSONFallbackEncoder,
    format_exc_info,
)
from structlog.threadlocal import wrap_dict


@pytest.fixture
def event_dict():
    class A(object):
        def __repr__(self):
            return '<A(\o/)>'

    return {'a': A(), 'b': [3, 4], 'x': 7, 'y': 'test', 'z': (1, 2)}


class TestKeyValueRenderer(object):
    def test_sort_keys(self, event_dict):
        assert (
            r"a=<A(\o/)> b=[3, 4] x=7 y='test' z=(1, 2)" ==
            KeyValueRenderer(sort_keys=True)(None, None, event_dict)
        )

    def test_order_complete(self, event_dict):
        assert (
            r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" ==
            KeyValueRenderer(key_order=['y', 'b', 'a', 'z', 'x'])
            (None, None, event_dict)
        )

    def test_order_missing(self, event_dict):
        """
        Missing keys get rendered as None.
        """
        assert (
            r"c=None y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" ==
            KeyValueRenderer(key_order=['c', 'y', 'b', 'a', 'z', 'x'])
            (None, None, event_dict)
        )

    def test_order_extra(self, event_dict):
        """
        Extra keys get sorted if sort_keys=True.
        """
        event_dict['B'] = 'B'
        event_dict['A'] = 'A'
        assert (
            r"c=None y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7 A='A' B='B'" ==
            KeyValueRenderer(key_order=['c', 'y', 'b', 'a', 'z', 'x'],
                             sort_keys=True)
            (None, None, event_dict)
        )

    def test_random_order(self, event_dict):
        rv = KeyValueRenderer()(None, None, event_dict)
        assert isinstance(rv, str)


class TestJSONRenderer(object):
    def test_renders_json(self, event_dict):
        assert (
            r'{"a": "<A(\\o/)>", "b": [3, 4], "x": 7, "y": "test", "z": '
            r'[1, 2]}'
            == JSONRenderer(sort_keys=True)(None, None, event_dict)
        )

    def test_FallbackEncoder_handles_ThreadLocalDictWrapped_dicts(self):
        s = json.dumps(wrap_dict(dict)({'a': 42}),
                       cls=_JSONFallbackEncoder)
        assert '{"a": 42}' == s

    def test_FallbackEncoder_falls_back(self):
        s = json.dumps({'date': datetime.date(1980, 3, 25)},
                       cls=_JSONFallbackEncoder,)

        assert '{"date": "datetime.date(1980, 3, 25)"}' == s


class TestTimeStamper(object):
    def test_disallowsNonUTCUNIXTimestamps(self):
        with pytest.raises(ValueError) as e:
            TimeStamper(utc=False)
        assert 'UNIX timestamps are always UTC.' == e.value.args[0]

    def test_insertsUTCUNIXTimestampByDefault(self):
        ts = TimeStamper()
        d = ts(None, None, {})
        # freezegun doesn't work with time.gmtime :(
        assert isinstance(d['timestamp'], int)

    @freeze_time('1980-03-25 16:00:00')
    def test_local(self):
        ts = TimeStamper(fmt='iso', utc=False)
        d = ts(None, None, {})
        assert '1980-03-25T16:00:00' == d['timestamp']

    @freeze_time('1980-03-25 16:00:00')
    def test_formats(self):
        ts = TimeStamper(fmt='%Y')
        d = ts(None, None, {})
        assert '1980' == d['timestamp']

    @freeze_time('1980-03-25 16:00:00')
    def test_adds_Z_to_iso(self):
        ts = TimeStamper(fmt='iso', utc=True)
        d = ts(None, None, {})
        assert '1980-03-25T16:00:00Z' == d['timestamp']


class TestFormatExcInfo(object):
    def test_formats_tuple(self, monkeypatch):
        monkeypatch.setattr(structlog.processors,
                            '_format_exception',
                            lambda exc_info: exc_info)
        d = format_exc_info(None, None, {'exc_info': (None, None, 42)})
        assert {'exception': (None, None, 42)} == d

    def test_gets_exc_info_on_bool(self):
        # monkeypatching sys.exc_info makes currently py.test return 1 on
        # success.
        try:
            raise ValueError('test')
        except ValueError:
            d = format_exc_info(None, None, {'exc_info': True})
        assert 'exc_info' not in d
        assert 'raise ValueError(\'test\')\nValueError: test' in d['exception']


class TestUnicodeEncoder(object):
    def test_encodes(self):
        ue = UnicodeEncoder()
        assert {'foo': b'b\xc3\xa4r'} == ue(None, None, {'foo': u('b\xe4r')})

    def test_passes_arguments(self):
        ue = UnicodeEncoder('latin1', 'xmlcharrefreplace')
        assert {'foo': b'&#8211;'} == ue(None, None, {'foo': u('\u2013')})
